/* mcuf.c
 * converter for BLINKENmini movies
 *  - converts movies between different file formats
 * Copyright (C) 2002 sphaera & 1stein (blinkenmini@schuermans.info, https://blinkenmini.schuermans.info/)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <malloc.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
       
#include "movie.h"
#include "mcuf.h"

//send frames using MCU-frames
//UdpSocket must be an nonblocking and connected UDP-socket
int McufSend( struct tMovie * pMovie, int UdpSocket )
{
	struct tFrame * pFrame;
	unsigned int FrameNo, Len, I, C, X, Y;
	unsigned char * pBuffer;

	//process all frames
	FrameNo = 0;
	for( pFrame = pMovie->pFirstFrame; pFrame != NULL; pFrame = pFrame->pNextFrame )
	{
		//print info message
		printf( "McufSend: sending frame %d...\n", FrameNo );

		//allocate buffer for packet
		Len = 12 + pFrame->SizeC * pFrame->SizeX * pFrame->SizeY;
		pBuffer = (unsigned char *)malloc( Len );
		if( pBuffer == NULL )
		{
			printf( "McufSend: could not allocate buffer for UDP-packet\n" );
			return -1;
		}

		//assemble packet
		pBuffer[0] = 0x23; //magic
		pBuffer[1] = 0x54;
		pBuffer[2] = 0x26;
		pBuffer[3] = 0x66;
		*(unsigned short *)(pBuffer + 4) = htons( pFrame->SizeY ); //height
		*(unsigned short *)(pBuffer + 6) = htons( pFrame->SizeX ); //width
		*(unsigned short *)(pBuffer + 8) = htons( pFrame->SizeC ); //channels
		*(unsigned short *)(pBuffer + 10) = htons( 0xFF ); //maximum value
		I = 12; //pixels
		for( Y = 0; Y < pFrame->SizeY; Y++ )
		for( X = 0; X < pFrame->SizeX; X++ )
		for( C = 0; C < pFrame->SizeC; C++ )
		{
			//put pixel into buffer
			pBuffer[I] = pFrame->pPixel[(Y * pFrame->SizeX + X) * pFrame->SizeC + C];
			I++;
		}

		//send frame
		send( UdpSocket, pBuffer, Len, 0 );
		//free buffer
		free( pBuffer );

		//wait for the requested time
		usleep( pFrame->Duration * 1000 );

		//count frames
		FrameNo++;
	} //for( pFrame ...

	//print info message
	printf( "McufSend: %d frames successfully sent\n", FrameNo );

	return 0;
}

